/*     karte_cgi (map_cgi) is interactively calculating maps  */
/*     via a web-interface  */

/*     Copyright (C) 2001  Marcus Lindemann (ml@ma3s.de) */

/*     This program is free software; you can redistribute it and/or modify */
/*     it under the terms of the GNU General Public License version 2 as */
/*     published by the Free Software Foundation. */

/*     This program is distributed in the hope that it will be useful, */
/*     but WITHOUT ANY WARRANTY; without even the implied warranty of */
/*     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the */
/*     GNU General Public License for more details. */

/*     You should have received a copy of the GNU General Public License */
/*     along with this program (file LICENSE); if not, write to the */
/*     Free Software Foundation, Inc., 59 Temple Place, Suite 330,  */
/*     Boston, MA  02111-1307  USA */

#ifndef __KARTE_H__
#define __KARTE_H__

#include "init.h"

#include <stdio.h>
#include <string.h>
#include <fcntl.h>
#include <sys/stat.h>
#include <sys/types.h>
#include <unistd.h>
#include <time.h>               //id calculation
#include <stdlib.h>

#include <gd.h>                 //libgd for image creation
#include <projects.h>           //proj-4.4.3 map projections lib
#include <cgic.h>               //cgic c lib for processing cgi

#define BIT32 int		//os specific
#define BIT16 short		//os specific

typedef struct
{
  BIT32 magic;
  BIT32 segment_index_address;
  BIT32 segment_index_count;
} Header;

typedef struct 
{
  BIT32 maxlat;
  BIT32 minlat;
  BIT32 maxlong;
  BIT32 minlong;
  BIT32 segment_address;
  BIT32 continent;
  BIT32 category;
  BIT32 type;
} Segment_index;

typedef struct 
{
  BIT32 orgx;
  BIT32 orgy;
  BIT32 nstrokes;
} Segment_header;

typedef struct
{
  BIT32 dx;
  BIT32 dy;
} Stroke;

typedef struct 
{
    double lam;
    double phi;
} CO;

typedef struct
{
    double x;
    double y;
} PT;

typedef struct
{
  BIT32 x;
  BIT32 y;
} Point;

typedef struct
{
  CO min;
  CO max;
} RANGE;

typedef struct      // image content for vectormap
{
  BIT32 continent;  // bitmap continent
  BIT32 category;   // bitmap category
  BIT32 type[9];    // bitmap type[category]
  BIT32 cil[16];    // mapping array [0,1] category[type]
  BIT32 riv[14];
  BIT32 bdy[5];
} Content; 

typedef struct
{
    char name[77]; // projection name parameter like "proj=ortho"
    int scale;     // scale to fit projection into image at zoom=1
    int wh;        // width/high 
    int pl;        // pole line, yes or no (1 or 0)
} PROJ;

typedef struct
{
    BIT32 width;   // image width
    BIT32 high;    // image high
    BIT32 scale;   // scale obtained by projection
    BIT32 zoom;    
    RANGE range;   // Range (Lon, Lat: min/max) covered Lon -180 - 180
                   // Lat -90 - 90
    Point offset;  // imagine a huge map of the world an you are looking at a 
                   // clipping at offset
    CO center;     // geographic center of your clipping
    gdImagePtr im; // image pointer
    PJ *pj;        // projection pointer
    int proj;      // projection number in pr
    PROJ pr[9];    // array of init values for projections
} IM;

typedef struct
{
  int z;           // factor for zoom
  int opt[3];      // options (color, turn,...)
  int colortbl[5][16]; // colortable
} VW;

// main
void standard_out();
void karte_out();

// map
int map(char *, IM *, VW *);

Point transform(CO *, IM *);
int image_out(char *, gdImagePtr);
void pt_print(Point, Segment_index *, IM *, VW *);

// map_init
int trans_init(IM *);
int offset_init(IM *);

// vworld
int vwseg_in(Segment_index *, Content *, IM *, VW *);
int vwdata(Content *, IM *, VW *);

// vw_init
void range_init(IM *);
void con_init(Content *);
int vw_init(IM *, VW *);
void pr_init(IM *);

// cgi_init
void im_init(IM *, IM *, VW *);
void pt_init(Point *);
void id(char *);

// center
BIT32 center(Point *, IM *, IM *);

// cgi_out
int cgi_out(IM *, VW *, Content *, char *);
char *sel(int, BIT32 *);
char *msel(int, int);
char *checked(int);

// helper
void html_head();
void html_foot();
double min(double, double);
double max(double, double);

// route
int route(IM *, VW *);

// deco
int deco(IM *, VW *);

#endif /* __KARTE_H__ */
