/*     karte_cgi (map_cgi) is interactively calculating maps  */
/*     via a web-interface  */

/*     Copyright (C) 2001  Marcus Lindemann (ml@ma3s.de) */

/*     This program is free software; you can redistribute it and/or modify */
/*     it under the terms of the GNU General Public License version 2 as */
/*     published by the Free Software Foundation. */

/*     This program is distributed in the hope that it will be useful, */
/*     but WITHOUT ANY WARRANTY; without even the implied warranty of */
/*     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the */
/*     GNU General Public License for more details. */

/*     You should have received a copy of the GNU General Public License */
/*     along with this program (file LICENSE); if not, write to the */
/*     Free Software Foundation, Inc., 59 Temple Place, Suite 330,  */
/*     Boston, MA  02111-1307  USA */

#include "karte.h"

int map(char *i_id, IM *img, VW *vw)
{
    Content con;

    trans_init(img);
    vw_init(img, vw);
    offset_init(img);
    con_init(&con);
    range_init(img);

    vwdata(&con, img, vw); // processing of world map data
    if(vw->opt[1]){route(img, vw);} // turn berlin-sydney2000 ...
    if(vw->opt[2]){deco(img, vw);}  // grid

    image_out(i_id, img->im);
    pj_free(img->pj);

    cgi_out(img, vw, &con, i_id);
    
    return 0;
}

int image_out(char *i_id, gdImagePtr im)
{
    FILE *OUT; //output file
    char name[256];

    sprintf(name, "%s%s.png", IMG_PREFIX, i_id);
    if (!(OUT=fopen(name,"wb"))) {
      fprintf(stderr,"Couldn't open output file.\n");
      exit (-1);
    }
    gdImagePng(im, OUT);
    fclose(OUT);
    gdImageDestroy(im);
    return 0;
}

Point transform(CO *dat, IM *img)
{
    Point pt;
    projUV p;

    p.u = DEG_TO_RAD*dat->lam;
    p.v = DEG_TO_RAD*dat->phi;
    p=pj_fwd(p, img->pj);
    pt.x=(BIT32)(img->width/2+p.u*img->zoom*img->scale-img->offset.x+0.5);
    pt.y=(BIT32)(img->high/2-p.v*img->zoom*img->scale+img->offset.y+0.5);

    return pt;
}

void pt_print(Point pt, Segment_index *seg, IM *img, VW *vw)
{
    const int m[9] = {0,1,2,0,3,0,0,0,4}; // mapping table
    int color;

    color=vw->colortbl[m[seg->category]][seg->type]; 
    gdImageSetPixel(img->im, pt.x, pt.y, color);
}

