/*     karte_cgi (map_cgi) is interactively calculating maps  */
/*     via a web-interface  */

/*     Copyright (C) 2001  Marcus Lindemann (ml@ma3s.de) */

/*     This program is free software; you can redistribute it and/or modify */
/*     it under the terms of the GNU General Public License version 2 as */
/*     published by the Free Software Foundation. */

/*     This program is distributed in the hope that it will be useful, */
/*     but WITHOUT ANY WARRANTY; without even the implied warranty of */
/*     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the */
/*     GNU General Public License for more details. */

/*     You should have received a copy of the GNU General Public License */
/*     along with this program (file LICENSE); if not, write to the */
/*     Free Software Foundation, Inc., 59 Temple Place, Suite 330,  */
/*     Boston, MA  02111-1307  USA */

#include "karte.h"

void range_init(IM *img)
{
    img->range.min.lam=180;
    img->range.min.phi=90;
    img->range.max.lam=-180;
    img->range.max.phi=-90;
}

void con_init(Content *con)
{
    int i, invalid;

    char *choices_cil[]={"0", "1", "2", "3", "4", "5", "6", "7", "8", "9", 
                         "10", "11", "12", "13", "14", "15"};
    char *choices_riv[]={"0", "1", "2", "3", "4", "5", "6", "7", "8", "9", 
                         "10", "11", "12", "13"};
    char *choices_bdy[]={"0", "1", "2", "3", "4"};

    // init
    con->continent=0xFFFFFFFF; // all continents -> world
    con->category=0x00000000;

    con->type[(0x00000001 << (PBY-1))]=0x00000000;
    con->type[(0x00000001 << (RIV-1))]=0x00000000;
    con->type[(0x00000001 << (BDY-1))]=0x00000000;
    con->type[(0x00000001 << (CIL-1))]=0x00000000;



    if(cgiFormSuccess == cgiFormSelectMultiple( "cil", choices_cil, 16,
       			  		          con->cil, &invalid)){
        con->category|=(0x00000001 << (CIL-1)); //coastline
	for(i=1; i<= 15; i++){
	    if(con->cil[i]){
                con->type[(0x00000001 << (CIL-1))]|=(0x00000001<<i);
	    }
	}
    }

    if(cgiFormSuccess == cgiFormSelectMultiple( "riv", choices_riv, 14,
						  con->riv, &invalid)){
        con->category|=(0x00000001 << (RIV-1)); //rivers, lakes, ...
	for(i=1; i<= 13; i++){
	    if(con->riv[i]){
                con->type[(0x00000001 << (RIV-1))]|=(0x00000001<<i);
	    }
	}
    }

    if(cgiFormSuccess == cgiFormSelectMultiple( "bdy", choices_bdy, 5,
						 con->bdy, &invalid)){  
	for(i=1; i<= 4; i++){
	    if(con->bdy[i] && i<4){
                con->category|=(0x00000001 << (BDY-1)); //boundary
                con->type[(0x00000001 << (BDY-1))]|=(0x00000001<<i);
	    }
            else if(con->bdy[4]){
                con->category|=(0x00000001 << (PBY-1)); //US boundary
                con->type[(0x00000001 << (PBY-1))]|=0xffffffff;
	    }
	}
    }
}

int vw_init(IM *img, VW *vw)
{
    char *values[]={"0", "1", "2"};
    int i, black, invalid;

    // init options
    cgiFormCheckboxMultiple( "opt", values, 3, vw->opt, &invalid);

    if(vw->opt[0]){
    // init colors
    vw->colortbl[PBY][1]= gdImageColorResolve(img->im, 255, 225, 200);
    vw->colortbl[PBY][3]= gdImageColorResolve(img->im, 255, 225, 200); 

    vw->colortbl[RIV][1]= gdImageColorResolve(img->im, 0, 0, 205);
    vw->colortbl[RIV][2]= gdImageColorResolve(img->im, 0, 0, 205); 
    vw->colortbl[RIV][3]= gdImageColorResolve(img->im, 28, 134, 238); 
    vw->colortbl[RIV][4]= gdImageColorResolve(img->im, 30, 144, 255);
    vw->colortbl[RIV][5]= gdImageColorResolve(img->im, 135, 206, 250); 
    vw->colortbl[RIV][6]= gdImageColorResolve(img->im, 0, 191, 255); 
    vw->colortbl[RIV][7]= gdImageColorResolve(img->im, 0, 191, 255); 
    vw->colortbl[RIV][8]= gdImageColorResolve(img->im, 0, 191, 255); 
    vw->colortbl[RIV][10]= gdImageColorResolve(img->im, 0, 0, 128); 
    vw->colortbl[RIV][11]= gdImageColorResolve(img->im, 25, 25, 112); 
    vw->colortbl[RIV][13]= gdImageColorResolve(img->im, 119, 136, 153); 

    vw->colortbl[BDY][1]= gdImageColorResolve(img->im, 255, 222, 173);  
    vw->colortbl[BDY][2]= gdImageColorResolve(img->im, 250, 240, 230);
    vw->colortbl[BDY][3]= gdImageColorResolve(img->im, 250, 235, 215);

    vw->colortbl[CIL][1]= gdImageColorResolve(img->im, 139, 121, 94); 
    vw->colortbl[CIL][2]= gdImageColorResolve(img->im, 139, 121, 94); 
    vw->colortbl[CIL][3]= gdImageColorResolve(img->im, 205, 179, 139); 
    vw->colortbl[CIL][4]= gdImageColorResolve(img->im, 205, 175, 149);
    vw->colortbl[CIL][6]= gdImageColorResolve(img->im, 30, 144, 255);
    vw->colortbl[CIL][7]= gdImageColorResolve(img->im, 99, 184, 255);
    vw->colortbl[CIL][8]= gdImageColorResolve(img->im, 127, 255, 0);
    vw->colortbl[CIL][9]= gdImageColorResolve(img->im, 245, 245, 245);
    vw->colortbl[CIL][10]= gdImageColorResolve(img->im, 245, 245, 245);
    vw->colortbl[CIL][13]= gdImageColorResolve(img->im, 0, 206, 209);
    vw->colortbl[CIL][14]= gdImageColorResolve(img->im, 0, 229, 238);
    vw->colortbl[CIL][15]= gdImageColorResolve(img->im, 0, 191, 255);
    }
    else{
	black = gdImageColorResolve(img->im, 0, 0, 0);
	for(i=1; i<=15; i++){
	    vw->colortbl[CIL][i]= black;
	} 
	for(i=1; i<=3; i++){
	    vw->colortbl[BDY][i]= black;
	}
	for(i=1; i<=13; i++){
	    vw->colortbl[RIV][i]= black;
	}
        vw->colortbl[PBY][1]= black;
        vw->colortbl[PBY][3]= black;
    }

    return 0;
}

void pr_init(IM *img)
{
    strcpy(img->pr[0].name, "proj=moll");
    img->pr[0].scale=63;
    img->pr[0].wh=2;
    img->pr[0].pl=0;
    strcpy(img->pr[1].name, "proj=ortho");
    img->pr[1].scale=90;
    img->pr[1].wh=1;
    img->pr[1].pl=0;
    strcpy(img->pr[2].name, "proj=sinu");
    img->pr[2].scale=57;
    img->pr[2].wh=2;
    img->pr[2].pl=0;
    strcpy(img->pr[3].name, "proj=merc");
    img->pr[3].scale=30;
    img->pr[3].wh=1;
    img->pr[3].pl=1;
    strcpy(img->pr[4].name, "proj=tmerc");
    img->pr[4].scale=30;
    img->pr[4].wh=1;
    img->pr[4].pl=0;
    strcpy(img->pr[5].name, "proj=gnom");
    img->pr[5].scale=20;
    img->pr[5].wh=1;
    img->pr[5].pl=0;
    strcpy(img->pr[6].name, "proj=eck4");
    img->pr[6].scale=67;
    img->pr[6].wh=2;
    img->pr[6].pl=1;
    strcpy(img->pr[7].name, "proj=eck6");
    img->pr[7].scale=65;
    img->pr[7].wh=2;
    img->pr[7].pl=1;
    strcpy(img->pr[8].name, "proj=vandg");
    img->pr[8].scale=32;
    img->pr[8].wh=1;
    img->pr[8].pl=0;
}
