/*     karte_cgi (map_cgi) is interactively calculating maps  */
/*     via a web-interface  */

/*     Copyright (C) 2001  Marcus Lindemann (ml@ma3s.de) */

/*     This program is free software; you can redistribute it and/or modify */
/*     it under the terms of the GNU General Public License version 2 as */
/*     published by the Free Software Foundation. */

/*     This program is distributed in the hope that it will be useful, */
/*     but WITHOUT ANY WARRANTY; without even the implied warranty of */
/*     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the */
/*     GNU General Public License for more details. */

/*     You should have received a copy of the GNU General Public License */
/*     along with this program (file LICENSE); if not, write to the */
/*     Free Software Foundation, Inc., 59 Temple Place, Suite 330,  */
/*     Boston, MA  02111-1307  USA */

#include "karte.h"

// test if segment seg has to be included by choosen seg 
int vwseg_in(Segment_index *seg, Content *con, IM *img, VW *vw)
{
    CO co_max, co_min; 
    Point pt_max, pt_min;

    if((seg->continent & con->continent) 
        && (seg->category & con->category)  
        && ((0x00000001 << seg->type) & con->type[seg->category])) 
    {
	co_max.lam=(seg->maxlong)/3600.0;
        co_max.phi=(seg->maxlat)/3600.0;
        co_min.lam=(seg->minlong)/3600.0;
        co_min.phi=(seg->minlat)/3600.0;
	pt_max=transform(&co_max,img);
        pt_min=transform(&co_min,img);

        if (  (pt_max.x>=0 && pt_min.x<=img->width
	    && pt_max.y<=img->high && pt_min.y>=0))
        { 
	  // save the range covered  
          img->range.min.lam=min(img->range.min.lam, co_min.lam);
          img->range.min.phi=min(img->range.min.phi, co_min.phi);
          img->range.max.lam=max(img->range.max.lam, co_max.lam);
          img->range.max.phi=max(img->range.max.phi, co_max.phi); 

          // whole segment into one point? -> print out this one point 
          if ((pt_max.x==pt_min.x)&&(pt_max.y==pt_min.y)){
	      pt_print(pt_max, seg, img, vw); 
              return 0; 
          }

	  return 1;
        }
    }
    return 0;  
}  

// read data from vector file, output data in choosen range, type
int vwdata(Content *con, IM *img, VW *vw)
{
  CO co;
  Header header_data;
  Segment_index *seg, *dir;
  Stroke *buf, *str;
  Segment_header segment_header_data;

  int DATA; //input Data - vector file

  DATA=open(EARTH, O_RDONLY, 0);
  if (DATA == -1) {
    fprintf(stderr,"Couldn't open input vector file.\n");
    exit (-1);
  }

  read(DATA, &header_data, sizeof(header_data));
  if (header_data.magic != HEADER_MAGIC) {
    fprintf(stderr, "The magic was not valid for the input file.\n");
    fprintf(stderr, "This could be an endian problem, or you\n");
    fprintf(stderr, "could be pointing to the wrong file.\n");
    exit(-2);
  }

  dir=(Segment_index *) 
      malloc(header_data.segment_index_count*sizeof(Segment_index));

  lseek(DATA, sizeof(header_data), SEEK_SET);
  read(DATA, dir, header_data.segment_index_count*sizeof(Segment_index));

  for (seg=dir; seg<dir+header_data.segment_index_count; seg++) {

    // segment choosen in desired range? (continent, category, long, lat,..)
    if (vwseg_in(seg, con, img, vw)){
      lseek(DATA, seg->segment_address, SEEK_SET);
      read(DATA, &segment_header_data, sizeof(segment_header_data)); 
      co.lam=(segment_header_data.orgx/3600.0);
      co.phi=(segment_header_data.orgy/3600.0);
      pt_print(transform(&co, img), seg, img, vw);
      
      buf=(Stroke *)malloc(segment_header_data.nstrokes*sizeof(Stroke));
      read(DATA, buf, segment_header_data.nstrokes*sizeof(Stroke));

      for (str=buf; str<buf+segment_header_data.nstrokes; str++) {
        co.lam=co.lam+(str->dx/3600.0);
        co.phi=co.phi+(str->dy/3600.0);
        if (img->zoom<=CUT){
	  // trimming data masses: zoom > CUT -> printing all points
          if ((str-buf)%(CUT*2/img->zoom)==0){
            pt_print(transform(&co, img), seg, img, vw);
          }
        }
        else pt_print(transform(&co, img), seg, img, vw);
      }
      free(buf);       
    }
  }
  free(dir);
  close(DATA);
  return 0;
}

